//+-------------------------------------------------------------------------------+
//|                                                                iCFractals.mq5 |
//| Copyright 2009-2020, MetaQuotes Software Corp.; 2012, 2021-2022, x572intraday |
//|                                                           http://www.mql5.com |
//+-------------------------------------------------------------------------------+
#property copyright "2009-2020, MetaQuotes Software Corp.; 2012, 2021-2022, x572intraday"
#property link      "http://www.mql5.com"
#property version   "1.99"

//--- indicator settings
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2
#property indicator_type1   DRAW_ARROW
#property indicator_type2   DRAW_ARROW
#property indicator_color1  Gray
#property indicator_color2  Gray
#property indicator_label1  "Fractal Up"
#property indicator_label2  "Fractal Down"
//--- input parameters
input uchar VHPERIODS; // virtual hyperperiods
//--- indicator buffers
double ExtUpperBuffer[];
double ExtLowerBuffer[];
//--- 10 pixels upper from high price
char   ExtArrowShift=-10;
//--- bars count
ushort BarsCount=350;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,ExtUpperBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtLowerBuffer,INDICATOR_DATA);

   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//--- sets first bar from what index will be drawn
//   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,Bars(_Symbol,_Period)-BarsCount);
//   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,Bars(_Symbol,_Period)-BarsCount);

   PlotIndexSetInteger(0,PLOT_ARROW,217);
   PlotIndexSetInteger(1,PLOT_ARROW,218);
//--- arrow shifts when drawing
   PlotIndexSetInteger(0,PLOT_ARROW_SHIFT,ExtArrowShift);
   PlotIndexSetInteger(1,PLOT_ARROW_SHIFT,-ExtArrowShift);
//--- sets drawing line empty value--
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//--- initialization done
  }
//+------------------------------------------------------------------+
//|  Fractals on 5 bars                                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   ENUM_TIMEFRAMES period=!VHPERIODS ? _Period : PERIOD_MN1;

   int calc=Bars(_Symbol,period);
   double high_VHMN[],low_VHMN[];
   int rt;

   if(VHPERIODS)
   {
      int high_tail_extremum_index=0;
      int high_head_extremum_index=0;
      int low_tail_extremum_index=0;
      int low_head_extremum_index=0;

      datetime FirstDateForMN1=(datetime)SeriesInfoInteger(_Symbol,period,SERIES_FIRSTDATE);
      datetime LastDateForMN1=(datetime)SeriesInfoInteger(_Symbol,period,SERIES_LASTBAR_DATE);

      int FDFMN1y=(int)StringSubstr((string)FirstDateForMN1,0,4);
      int FDFMN1m=(int)StringSubstr((string)FirstDateForMN1,5,2);
      int LDFMN1y=(int)StringSubstr((string)LastDateForMN1,0,4);
      int LDFMN1m=(int)StringSubstr((string)LastDateForMN1,5,2);

      int right_boundary_of_the_last_full_period=(int)floor((double)LDFMN1y/VHPERIODS*12)*VHPERIODS/12;
      int total_months_till_last_full_period=right_boundary_of_the_last_full_period*12;
      int the_rest_of_the_months=FDFMN1y*12+(FDFMN1m-1);
      int total_months_till_now=LDFMN1y*12+LDFMN1m;

      int tail_of_the_first_period=(total_months_till_last_full_period-the_rest_of_the_months)%VHPERIODS;
      int head_of_the_last_period=total_months_till_now-right_boundary_of_the_last_full_period*12;
      int body_of_full_periods=(calc-tail_of_the_first_period-head_of_the_last_period);

      int vhp=0;
      int ar=0;
      if(tail_of_the_first_period) ar++;
      if(head_of_the_last_period) ar++;
      if(body_of_full_periods) ar+=body_of_full_periods/VHPERIODS;
      if(ar<5) return(0);

      ArrayResize(high_VHMN,ar);
      ArrayResize(low_VHMN,ar);

      high_tail_extremum_index=ArrayMaximum(high,0,tail_of_the_first_period); // iHighest?
      low_tail_extremum_index=ArrayMinimum(low,0,tail_of_the_first_period); // iLowest?

      if(tail_of_the_first_period)
      {
         high_VHMN[0]=iHigh(_Symbol,period,calc-high_tail_extremum_index-1);
         low_VHMN[0]=iLow(_Symbol,period,calc-low_tail_extremum_index-1);
      }
      else
         vhp=VHPERIODS;

      high_head_extremum_index=ArrayMaximum(high,calc-head_of_the_last_period,head_of_the_last_period); // iHighest?
      low_head_extremum_index=ArrayMinimum(low,calc-head_of_the_last_period,head_of_the_last_period); // iLowest?

      if(head_of_the_last_period)
      {
         high_VHMN[ar-1]=iHigh(_Symbol,period,high_head_extremum_index);
         low_VHMN[ar-1]=iLow(_Symbol,period,low_head_extremum_index);
      }

      for(int lbound=tail_of_the_first_period+VHPERIODS-vhp; lbound<calc-head_of_the_last_period; lbound+=VHPERIODS)
      {
         high_VHMN[lbound/VHPERIODS]=iHigh(_Symbol,period,iHighest(NULL,period,MODE_HIGH,VHPERIODS,calc-lbound));
         low_VHMN[lbound/VHPERIODS]=iLow(_Symbol,period,iLowest(NULL,period,MODE_LOW,VHPERIODS,calc-lbound));
      }
   }
//+------------------------------------------------------------------+
   rt=VHPERIODS ? ArraySize(high_VHMN) : calc;

   if(!VHPERIODS && rt<5) // doesn't work without !VHPERIODS
      return(0);

   int start;
//--- clean up arrays
   if(prev_calculated<7)
   {
      if(rt>=BarsCount)
         start=rt-BarsCount+2;
      else
         start=2;
      ArrayInitialize(ExtUpperBuffer,EMPTY_VALUE);
      ArrayInitialize(ExtLowerBuffer,EMPTY_VALUE);
   }
   else
      if(rt>=BarsCount)
         start=BarsCount-5;
      else
         start=rt-5;
//+------------------------------------------------------------------+
   //--- main cycle of calculations
   int incU=0;
   int incL=0;
   int upper_fractal_indices[], lower_fractal_indices[];

   if(!VHPERIODS)
   {
      for(int i=start; i<calc-3 && !IsStopped(); i++)
      {
         //--- Upper Fractal
         if(high[i]>high[i+1] && high[i]>high[i+2] && high[i]>=high[i-1] && high[i]>=high[i-2])
            ExtUpperBuffer[i]=high[i];
         else
            ExtUpperBuffer[i]=EMPTY_VALUE;

         //--- Lower Fractal
         if(low[i]<low[i+1] && low[i]<low[i+2] && low[i]<=low[i-1] && low[i]<=low[i-2])
            ExtLowerBuffer[i]=low[i];
         else
            ExtLowerBuffer[i]=EMPTY_VALUE;
      }
   }
   else
   {
      for(int i=start; i<calc-3 && !IsStopped(); i++)
      {
         //--- Upper Virtual Hyperfractal's preparation
         if(high[i]>high[i+1] && high[i]>high[i+2] && high[i]>=high[i-1] && high[i]>=high[i-2])
         {
            ArrayResize(upper_fractal_indices,incU+1);
            upper_fractal_indices[incU++]=i;
         }

         //--- Lower Virtual Hyperfractal's preparation
         if(low[i]<low[i+1] && low[i]<low[i+2] && low[i]<=low[i-1] && low[i]<=low[i-2])
         {
            ArrayResize(lower_fractal_indices,incL+1);
            lower_fractal_indices[incL++]=i;
         }
      }

      for(int i=start; i<rt-3 && !IsStopped(); i++)
      {
         //--- Upper Virtual Hyperfractal
         if(high_VHMN[i]>high_VHMN[i+1] && high_VHMN[i]>high_VHMN[i+2] &&
            high_VHMN[i]>=high_VHMN[i-1] && high_VHMN[i]>=high_VHMN[i-2])
            for(int iH=0; iH<ArraySize(upper_fractal_indices); iH++)
               if(high[upper_fractal_indices[iH]]==high_VHMN[i])
                  ExtUpperBuffer[upper_fractal_indices[iH]]=high_VHMN[i];

         //--- Lower Virtual Hyperfractal
         if(low_VHMN[i]<low_VHMN[i+1] && low_VHMN[i]<low_VHMN[i+2] &&
            low_VHMN[i]<=low_VHMN[i-1] && low_VHMN[i]<=low_VHMN[i-2])
            for(int iL=0; iL<ArraySize(lower_fractal_indices); iL++)
               if(low[lower_fractal_indices[iL]]==low_VHMN[i])
                  ExtLowerBuffer[lower_fractal_indices[iL]]=low_VHMN[i];
      }
   }
//--- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
//+------------------------------------------------------------------+
